/*
 * @(#)JAIAddConstPanel.java	8.1 99/12/21
 *
 * Copyright (c) 1998 Sun Microsystems, Inc. All Rights Reserved.
 *
 * Sun grants you ("Licensee") a non-exclusive, royalty free, license to use,
 * modify and redistribute this software in source and binary code form,
 * provided that i) this copyright notice and license appear on all copies of
 * the software; and ii) Licensee does not utilize the software in a manner
 * which is disparaging to Sun.
 *
 * This software is provided "AS IS," without a warranty of any kind. ALL
 * EXPRESS OR IMPLIED CONDITIONS, REPRESENTATIONS AND WARRANTIES, INCLUDING ANY
 * IMPLIED WARRANTY OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR
 * NON-INFRINGEMENT, ARE HEREBY EXCLUDED. SUN AND ITS LICENSORS SHALL NOT BE
 * LIABLE FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING
 * OR DISTRIBUTING THE SOFTWARE OR ITS DERIVATIVES. IN NO EVENT WILL SUN OR ITS
 * LICENSORS BE LIABLE FOR ANY LOST REVENUE, PROFIT OR DATA, OR FOR DIRECT,
 * INDIRECT, SPECIAL, CONSEQUENTIAL, INCIDENTAL OR PUNITIVE DAMAGES, HOWEVER
 * CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY, ARISING OUT OF THE USE OF
 * OR INABILITY TO USE SOFTWARE, EVEN IF SUN HAS BEEN ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGES.
 *
 * This software is not designed or intended for use in on-line control of
 * aircraft, air traffic, aircraft navigation or aircraft communications; or in
 * the design, construction, operation or maintenance of any nuclear
 * facility. Licensee represents and warrants that it will not use or
 * redistribute the Software for such purposes.
 */

import java.awt.*;
import java.awt.event.*;
import java.awt.image.renderable.ParameterBlock;
import java.awt.image.renderable.RenderedImageFactory;
import java.util.Hashtable;
import java.util.Vector;
import javax.media.jai.*;
import javax.swing.*;
import javax.swing.event.*;

public class JAIAddConstPanel extends JAIDemoPanel
    implements ChangeListener, ItemListener {

    int param1 = 0;
    int param2 = 0;
    int param3 = 0;

    JSlider p1Slider;
    JSlider p2Slider;
    JSlider p3Slider;

    public JAIAddConstPanel(Vector sourceVec) {
        super(sourceVec);
        masterSetup();
    }

    public String getDemoName() {
        return "AddConst";
    }


    public void makeControls(JPanel controls) {
        p1Slider = new JSlider(JSlider.HORIZONTAL, -255, 255, 0);
        p2Slider = new JSlider(JSlider.HORIZONTAL, -255, 255, 0);
        p3Slider = new JSlider(JSlider.HORIZONTAL, -255, 255, 0);

        Hashtable labels = new Hashtable();
        labels.put(new Integer(-255), new JLabel("-255"));
        labels.put(new Integer(0), new JLabel("0"));
        labels.put(new Integer(255), new JLabel("255"));
        p1Slider.setLabelTable(labels);
        p2Slider.setLabelTable(labels);
        p3Slider.setLabelTable(labels);

        p1Slider.setPaintLabels(true);
        p2Slider.setPaintLabels(true);
        p3Slider.setPaintLabels(true);
        
        p1Slider.addChangeListener(this);
        p2Slider.addChangeListener(this);
        p3Slider.addChangeListener(this);

        JPanel p1SliderPanel = new JPanel();
        p1SliderPanel.setLayout(new BoxLayout(p1SliderPanel, BoxLayout.X_AXIS));
        JLabel p1Label = new JLabel("Red");
        p1SliderPanel.add(p1Label);
        p1SliderPanel.add(p1Slider);

        JPanel p2SliderPanel = new JPanel();
        p2SliderPanel.setLayout(new BoxLayout(p2SliderPanel, BoxLayout.X_AXIS));
        JLabel p2Label = new JLabel("Green");
        p2Label.setPreferredSize(p1Label.getPreferredSize());
        p2SliderPanel.add(p2Label);
        p2SliderPanel.add(p2Slider);

        JPanel p3SliderPanel = new JPanel();
        p3SliderPanel.setLayout(new BoxLayout(p3SliderPanel, BoxLayout.X_AXIS));
        JLabel p3Label = new JLabel("Blue");
        p3Label.setPreferredSize(p1Label.getPreferredSize());
        p3SliderPanel.add(p3Label);
        p3SliderPanel.add(p3Slider);

        JPanel sliderPanel = new JPanel();
        sliderPanel.setLayout(new BoxLayout(sliderPanel, BoxLayout.Y_AXIS));
        sliderPanel.add(p1SliderPanel);
        sliderPanel.add(p2SliderPanel);
        sliderPanel.add(p3SliderPanel);

        controls.setLayout(new BorderLayout());
        controls.add("Center", sliderPanel);
    }

    public boolean supportsAutomatic() {
        return true;
    }

    public PlanarImage process() {
        PlanarImage im = getSource(0);
 
        ParameterBlock pb = new ParameterBlock();
        pb.addSource(im);
        double consts[] = {param1, 
                           param2,
                           param3};
        pb.add(consts);
        return JAI.create("addconst", pb, renderHints);
    }

    public void startAnimation() {
    }

    int sliderDelta = 5;

    public void animate() {
        int value = p1Slider.getValue();
        int newValue = value + sliderDelta;

        if (newValue < p1Slider.getMinimum() ||
            newValue > p1Slider.getMaximum()) {
            sliderDelta = -sliderDelta;
        }
        p1Slider.setValue(value + sliderDelta);
    }

    public void reset() {
        param1 = 0;
        param2 = 0;
        param3 = 0;
        p1Slider.setValue(param1);
        p2Slider.setValue(param2);
        p3Slider.setValue(param2);
    }

    public void stateChanged(ChangeEvent e) {
        JSlider source = (JSlider)e.getSource();
        int value = source.getValue();

        if (source == p1Slider) {
             param1 = value;
        } else if (source == p2Slider) {
             param2 = value;
        } else if (source == p3Slider) {
             param3 = value;
        }
        repaint();
    }

    public void itemStateChanged(ItemEvent e) {
    }
}
